@@ -1,5 +1,5 @@
-# parsetree.py
-# Copyright (C) 2006, 2007, 2008, 2009, 2010 Michael Bayer mike_mp@zzzcomputing.com
+# mako/parsetree.py
+# Copyright (C) 2006-2012 the Mako authors and contributors <see AUTHORS file>
 #
 # This module is part of Mako and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -8,63 +8,68 @@
 
 from mako import exceptions, ast, util, filters
 import re
-
+ 
 class Node(object):
     """base class for a Node in the parse tree."""
+
     def __init__(self, source, lineno, pos, filename):
         self.source = source
         self.lineno = lineno
         self.pos = pos
         self.filename = filename
-        
+ 
     @property
     def exception_kwargs(self):
         return {'source':self.source, 'lineno':self.lineno, 
                 'pos':self.pos, 'filename':self.filename}
-    
+ 
     def get_children(self):
         return []
-        
+ 
     def accept_visitor(self, visitor):
         def traverse(node):
             for n in node.get_children():
                 n.accept_visitor(visitor)
+
         method = getattr(visitor, "visit" + self.__class__.__name__, traverse)
         method(self)
 
 class TemplateNode(Node):
     """a 'container' node that stores the overall collection of nodes."""
-    
+ 
     def __init__(self, filename):
         super(TemplateNode, self).__init__('', 0, 0, filename)
         self.nodes = []
         self.page_attributes = {}
-        
+ 
     def get_children(self):
         return self.nodes
-        
+ 
     def __repr__(self):
         return "TemplateNode(%s, %r)" % (
                     util.sorted_dict_repr(self.page_attributes), 
                     self.nodes)
-        
+ 
 class ControlLine(Node):
     """defines a control line, a line-oriented python line or end tag.
-    
+ 
     e.g.::
 
         % if foo:
             (markup)
         % endif
-    
+ 
     """
 
+    has_loop_context = False
+
     def __init__(self, keyword, isend, text, **kwargs):
         super(ControlLine, self).__init__(**kwargs)
         self.text = text
         self.keyword = keyword
         self.isend = isend
-        self.is_primary = keyword in ['for','if', 'while', 'try']
+        self.is_primary = keyword in ['for', 'if', 'while', 'try', 'with']
+        self.nodes = []
         if self.isend:
             self._declared_identifiers = []
             self._undeclared_identifiers = []
@@ -73,22 +78,25 @@ class ControlLine(Node):
             self._declared_identifiers = code.declared_identifiers 
             self._undeclared_identifiers = code.undeclared_identifiers
 
+    def get_children(self):
+        return self.nodes
+
     def declared_identifiers(self):
         return self._declared_identifiers
 
     def undeclared_identifiers(self):
         return self._undeclared_identifiers
-        
+ 
     def is_ternary(self, keyword):
         """return true if the given keyword is a ternary keyword
         for this ControlLine"""
-        
+ 
         return keyword in {
             'if':set(['else', 'elif']),
             'try':set(['except', 'finally']),
             'for':set(['else'])
         }.get(self.keyword, [])
-        
+ 
     def __repr__(self):
         return "ControlLine(%r, %r, %r, %r)" % (
             self.keyword, 
@@ -99,29 +107,29 @@ class ControlLine(Node):
 
 class Text(Node):
     """defines plain text in the template."""
-    
+ 
     def __init__(self, content, **kwargs):
         super(Text, self).__init__(**kwargs)
         self.content = content
-        
+ 
     def __repr__(self):
         return "Text(%r, %r)" % (self.content, (self.lineno, self.pos))
-        
+ 
 class Code(Node):
     """defines a Python code block, either inline or module level.
-    
+ 
     e.g.::
 
         inline:
         <%
             x = 12
         %>
-    
+ 
         module level:
         <%!
             import logger
         %>
-    
+ 
     """
 
     def __init__(self, text, ismodule, **kwargs):
@@ -142,28 +150,28 @@ class Code(Node):
             self.ismodule, 
             (self.lineno, self.pos)
         )
-        
+ 
 class Comment(Node):
     """defines a comment line.
-    
+ 
     # this is a comment
-    
+ 
     """
-    
+ 
     def __init__(self, text, **kwargs):
         super(Comment, self).__init__(**kwargs)
         self.text = text
 
     def __repr__(self):
         return "Comment(%r, %r)" % (self.text, (self.lineno, self.pos))
-        
+ 
 class Expression(Node):
     """defines an inline expression.
-    
+ 
     ${x+y}
-    
+ 
     """
-    
+ 
     def __init__(self, text, escapes, **kwargs):
         super(Expression, self).__init__(**kwargs)
         self.text = text
@@ -188,18 +196,18 @@ class Expression(Node):
             self.escapes_code.args, 
             (self.lineno, self.pos)
         )
-        
+ 
 class _TagMeta(type):
     """metaclass to allow Tag to produce a subclass according to
     its keyword"""
-    
+ 
     _classmap = {}
-    
+ 
     def __init__(cls, clsname, bases, dict):
         if cls.__keyword__ is not None:
             cls._classmap[cls.__keyword__] = cls
             super(_TagMeta, cls).__init__(clsname, bases, dict)
-            
+ 
     def __call__(cls, keyword, attributes, **kwargs):
         if ":" in keyword:
             ns, defname = keyword.split(':')
@@ -217,41 +225,41 @@ class _TagMeta(type):
                 filename=kwargs['filename']
             )
         return type.__call__(cls, keyword, attributes, **kwargs)
-        
+ 
 class Tag(Node):
     """abstract base class for tags.
-    
+ 
     <%sometag/>
-    
+ 
     <%someothertag>
         stuff
     </%someothertag>
-    
+ 
     """
-    
+ 
     __metaclass__ = _TagMeta
     __keyword__ = None
-    
+ 
     def __init__(self, keyword, attributes, expressions, 
                         nonexpressions, required, **kwargs):
         """construct a new Tag instance.
-        
+ 
         this constructor not called directly, and is only called
         by subclasses.
-        
+ 
         :param keyword: the tag keyword
-        
+ 
         :param attributes: raw dictionary of attribute key/value pairs
-        
+ 
         :param expressions: a set of identifiers that are legal attributes, 
          which can also contain embedded expressions
-        
+ 
         :param nonexpressions: a set of identifiers that are legal 
          attributes, which cannot contain embedded expressions
-        
+ 
         :param \**kwargs:
          other arguments passed to the Node superclass (lineno, pos)
-        
+ 
         """
         super(Tag, self).__init__(**kwargs)
         self.keyword = keyword
@@ -265,13 +273,13 @@ class Tag(Node):
                 **self.exception_kwargs)
         self.parent = None
         self.nodes = []
-        
+ 
     def is_root(self):
         return self.parent is None
-        
+ 
     def get_children(self):
         return self.nodes
-        
+ 
     def _parse_attributes(self, expressions, nonexpressions):
         undeclared_identifiers = set()
         self.parsed_attributes = {}
@@ -299,9 +307,9 @@ class Tag(Node):
             elif key in nonexpressions:
                 if re.search(r'\${.+?}', self.attributes[key]):
                     raise exceptions.CompileException(
-                            "Attibute '%s' in tag '%s' does not allow embedded "
-                            "expressions"  % (key, self.keyword), 
-                            **self.exception_kwargs)
+                           "Attibute '%s' in tag '%s' does not allow embedded "
+                           "expressions"  % (key, self.keyword), 
+                           **self.exception_kwargs)
                 self.parsed_attributes[key] = repr(self.attributes[key])
             else:
                 raise exceptions.CompileException(
@@ -323,7 +331,7 @@ class Tag(Node):
                                     (self.lineno, self.pos), 
                                     self.nodes
                                 )
-        
+ 
 class IncludeTag(Tag):
     __keyword__ = 'include'
 
@@ -346,7 +354,7 @@ class IncludeTag(Tag):
                             difference(self.page_args.declared_identifiers)
         return identifiers.union(super(IncludeTag, self).
                                     undeclared_identifiers())
-    
+ 
 class NamespaceTag(Tag):
     __keyword__ = 'namespace'
 
@@ -357,13 +365,18 @@ class NamespaceTag(Tag):
                                         ('name','inheritable',
                                         'import','module'), 
                                         (), **kwargs)
-                                        
+ 
         self.name = attributes.get('name', '__anon_%s' % hex(abs(id(self))))
         if not 'name' in attributes and not 'import' in attributes:
             raise exceptions.CompileException(
-                            "'name' and/or 'import' attributes are required "
-                            "for <%namespace>", 
-                                **self.exception_kwargs)
+                "'name' and/or 'import' attributes are required "
+                "for <%namespace>",
+                **self.exception_kwargs)
+        if 'file' in attributes and 'module' in attributes:
+            raise exceptions.CompileException(
+                "<%namespace> may only have one of 'file' or 'module'",
+                **self.exception_kwargs
+            )
 
     def declared_identifiers(self):
         return []
@@ -379,16 +392,19 @@ class TextTag(Tag):
         self.filter_args = ast.ArgumentList(
                                     attributes.get('filter', ''), 
                                     **self.exception_kwargs)
-        
+ 
 class DefTag(Tag):
     __keyword__ = 'def'
 
     def __init__(self, keyword, attributes, **kwargs):
+        expressions = ['buffered', 'cached'] + [
+                c for c in attributes if c.startswith('cache_')]
+
+
         super(DefTag, self).__init__(
                 keyword, 
                 attributes, 
-                ('buffered', 'cached', 'cache_key', 'cache_timeout', 
-                    'cache_type', 'cache_dir', 'cache_url'), 
+                expressions, 
                 ('name','filter', 'decorator'), 
                 ('name',), 
                 **kwargs)
@@ -405,6 +421,16 @@ class DefTag(Tag):
                                 attributes.get('filter', ''), 
                                 **self.exception_kwargs)
 
+    is_anonymous = False
+    is_block = False
+
+    @property
+    def funcname(self):
+        return self.function_decl.funcname
+
+    def get_argument_expressions(self, **kw):
+        return self.function_decl.get_argument_expressions(**kw)
+
     def declared_identifiers(self):
         return self.function_decl.argnames
 
@@ -413,10 +439,71 @@ class DefTag(Tag):
         for c in self.function_decl.defaults:
             res += list(ast.PythonCode(c, **self.exception_kwargs).
                                     undeclared_identifiers)
-        return res + list(self.filter_args.\
+        return set(res).union(
+            self.filter_args.\
                             undeclared_identifiers.\
                             difference(filters.DEFAULT_ESCAPES.keys())
-                        )
+        ).union(
+            self.expression_undeclared_identifiers
+        )
+
+class BlockTag(Tag):
+    __keyword__ = 'block'
+
+    def __init__(self, keyword, attributes, **kwargs):
+        expressions = ['buffered', 'cached', 'args'] + [
+                 c for c in attributes if c.startswith('cache_')]
+
+        super(BlockTag, self).__init__(
+                keyword, 
+                attributes, 
+                expressions,
+                ('name','filter', 'decorator'), 
+                (), 
+                **kwargs)
+        name = attributes.get('name')
+        if name and not re.match(r'^[\w_]+$',name):
+            raise exceptions.CompileException(
+                               "%block may not specify an argument signature", 
+                               **self.exception_kwargs)
+        if not name and attributes.get('args', None):
+            raise exceptions.CompileException(
+                                "Only named %blocks may specify args",
+                                **self.exception_kwargs
+                                )
+        self.body_decl = ast.FunctionArgs(attributes.get('args', ''), 
+                                            **self.exception_kwargs)
+
+        self.name = name
+        self.decorator = attributes.get('decorator', '')
+        self.filter_args = ast.ArgumentList(
+                                attributes.get('filter', ''), 
+                                **self.exception_kwargs)
+
+
+    is_block = True
+
+    @property
+    def is_anonymous(self):
+        return self.name is None
+
+    @property
+    def funcname(self):
+        return self.name or "__M_anon_%d" % (self.lineno, )
+
+    def get_argument_expressions(self, **kw):
+        return self.body_decl.get_argument_expressions(**kw)
+
+    def declared_identifiers(self):
+        return self.body_decl.argnames
+
+    def undeclared_identifiers(self):
+        return (self.filter_args.\
+                            undeclared_identifiers.\
+                            difference(filters.DEFAULT_ESCAPES.keys())
+                ).union(self.expression_undeclared_identifiers)
+
+
 
 class CallTag(Tag):
     __keyword__ = 'call'
@@ -446,7 +533,7 @@ class CallNamespaceTag(Tag):
                     (), 
                     (), 
                     **kwargs)
-                    
+ 
         self.expression = "%s.%s(%s)" % (
                                 namespace, 
                                 defname, 
@@ -478,12 +565,13 @@ class PageTag(Tag):
     __keyword__ = 'page'
 
     def __init__(self, keyword, attributes, **kwargs):
+        expressions =   ['cached', 'args', 'expression_filter', 'enable_loop'] + [
+                    c for c in attributes if c.startswith('cache_')]
+
         super(PageTag, self).__init__(
                 keyword, 
                 attributes, 
-                ('cached', 'cache_key', 'cache_timeout', 
-                'cache_type', 'cache_dir', 'cache_url', 
-                'args', 'expression_filter'), 
+                expressions,
                 (), 
                 (), 
                 **kwargs)
@@ -495,5 +583,5 @@ class PageTag(Tag):
 
     def declared_identifiers(self):
         return self.body_decl.argnames
-        
-    
+ 
+ 
